import { arbres as importedData } from './arbres.js';

// Définition des dimensions de la visualisation.
const width = window.innerWidth;
const height = window.innerHeight;

const margin = {top:20, right:20, bottom:20, left:20};

// Création du canevas SVG.
const svg = d3.select("#arbre_svg")
  .attr("width", width)
  .attr("height", height);

// Ajout d'un groupe (g) pour contenir les éléments de la visualisation.
const g = svg.append("g")
  .attr("transform", `translate(${margin.left},${margin.top})`);


// Tri des arbres par accuracy dans l'ordre décroissant
//data.sort((a, b) => b.accuracy - a.accuracy);


// Configuration de la disposition de l'arbre.
const treeLayout = d3.tree().size(
  [0.6 * (height - margin.top - margin.bottom), 
  0.6 * (width - margin.left - margin.right)/3.5]);


// Fonction pour générer des arbres hiérarchiques à partir des données.
function generateTree(treeData, treeId) {
  const root = d3.hierarchy(treeData);
  root.each((node, i) => {
    node.id = `${treeId}-${i}`;
    node.data.accuracy = node.parent ? node.parent.data.accuracy : treeData.accuracy; 
  });
  return treeLayout(root);
}

// Fonction pour déterminer la couleur en fonction de l'accuracy.
function getColorByAccuracy(accuracy) {
  if (accuracy >= 0.8) {
    return "red";
  } else if (accuracy >= 0.6) {
    return "blue";
  } else {
    return "gray";
  }
}


// Variables pour ajuster les positions et tailles des arbres.
let xOffset = 0;
let maxHeight = 0;


// Facteur d'espacement vertical pour ajuster l'espace entre les arbres
const verticalSpacingFactor = 1.5;


//calculer le cercle circonscrit
function calculateBoundingCircle(tree) {
  const xCoords = tree.descendants().map(d => d.x);
  const yCoords = tree.descendants().map(d => d.y);
  const minX = Math.min(...xCoords);
  const maxX = Math.max(...xCoords);
  const minY = Math.min(...yCoords);
  const maxY = Math.max(...yCoords);
  const centerX = (minX + maxX) / 2;
  const centerY = (minY + maxY) / 2;
  const radius = Math.sqrt(Math.pow(maxX - minX, 2) + Math.pow(maxY - minY, 2)) / 2;
  return { centerX, centerY, radius };
}

// Conversion des données en arbres hiérarchiques.
const data = importedData.map((treeData, i) => generateTree(treeData, i + 1));

// Fonction pour basculer la classe "toggled" sur les nœuds ayant le même nom.
function toggleSameNodes(d) {
  // Trouver tous les nœuds portant le même nom dans tous les arbres
  const sameNodes = data.flatMap(tree => tree.descendants())
    .filter(node => node.data.name === d.data.name);

  // Basculer la classe "toggled" sur les nœuds trouvés
  svg.selectAll(".node")
    .filter(node => sameNodes.includes(node))
    .classed("toggled", sameNodes[0].data.toggled ? false : true);
}


const boundingCircles = data.map(tree => calculateBoundingCircle(tree));


let totalWidth = width;

// Facteur d'espacement pour ajuster l'espace entre les arbres
const spacingFactor = 1.2;

// Mettre à jour la largeur totale pour inclure le facteur d'espacement
totalWidth = totalWidth * spacingFactor;

// Création d'une échelle pour l'axe de l'accuracy.
const accuracyScale = d3.scaleOrdinal()
  .domain(["Groupe 1", "Groupe 2", "Groupe 3"])
  .range([0, (totalWidth - margin.left - margin.right) / 3, (totalWidth - margin.left - margin.right) * 2 / 3]);



// Fonction pour calculer la position X de chaque arbre en fonction de son accuracy
function calculateTreeXPosition(accuracy) {
  let xPos;
  if (accuracy >= 0.8) {
    xPos = totalWidth / 6;
  } else if (accuracy >= 0.6) {
    xPos = totalWidth / 2;
  } else {
    xPos = (5 * totalWidth) / 6;
  }
  return xPos;
}

// Variables pour ajuster les positions des arbres.
const yOffsetGroup1 = [];
const yOffsetGroup2 = [];
const yOffsetGroup3 = [];

//****************************************************
//****************************************************

// Boucle sur chaque arbre pour créer la visualisation.
data.forEach((tree, index) => {
  const treeWidths = tree.descendants().map(d => d.x);
  const maxTreeWidth = Math.max(...treeWidths);
  const treeHeights = tree.descendants().map(d => d.y);
  const maxTreeHeight = Math.max(...treeHeights);

  const treeXPosition = calculateTreeXPosition(tree.data.accuracy);
  
  // Calcul du décalage sur l'axe Y pour centrer chaque arbre.
  let yOffset;

  const boundingCircleRadius = boundingCircles[index].radius;
  const accumulatedRadius = (group) => group.reduce((acc, radius) => acc + radius, 0);
  
  if (tree.data.accuracy >= 0.8) {
    yOffset = accumulatedRadius(yOffsetGroup1) + yOffsetGroup1.length * (maxHeight + margin.bottom);
    yOffsetGroup1.push(boundingCircleRadius);
  } else if (tree.data.accuracy >= 0.6) {
    yOffset = accumulatedRadius(yOffsetGroup2) + yOffsetGroup2.length * (maxHeight + margin.bottom);
    yOffsetGroup2.push(boundingCircleRadius);
  } else {
    yOffset = accumulatedRadius(yOffsetGroup3) + yOffsetGroup3.length * (maxHeight + margin.bottom);
    yOffsetGroup3.push(boundingCircleRadius);
  }
  

  // Création d'un groupe (g) pour chaque arbre.
	const treeGroup = g.append("g")
  	.attr("transform", `translate(${treeXPosition - boundingCircles[index].centerX}, ${yOffset})`);
  
  // Dessiner le cercle circonscrit pour chaque arbre en utilisant la couleur de la racine.
  const boundingCircle = boundingCircles[index];
  const rootColor = getColorByAccuracy(tree.data.accuracy);

  treeGroup.append("circle")
    .attr("class", "bounding-circle")
    .attr("cx", boundingCircle.centerX)
    .attr("cy", boundingCircle.centerY)
    .attr("r", boundingCircle.radius)
    .attr("stroke", rootColor) // Utilise la couleur de la racine pour le contour du cercle.
    .attr("fill", rootColor) // Utilise la couleur de la racine pour le remplissage du cercle.
    .attr("opacity", 0.5); // Ajoute une opacité pour rendre le cercle semi-transparent.


    treeGroup.append("circle")
      .attr("class", "bounding-circle")
      .attr("cx", boundingCircle.centerX)
      .attr("cy", boundingCircle.centerY)
      .attr("r", boundingCircle.radius)
      .attr("stroke", "black")
      .attr("fill", "none");
  
  treeGroup.append("text")
  .attr("class", "accuracy-label")
  .attr("x", boundingCircle.centerX)
  .attr("y", 0) // Ajuste la position verticale des étiquettes
  .attr("text-anchor", "middle")
  .text(`${tree.data.accuracy.toFixed(2)}`)
  .attr("font-family", "Arial")
  .attr("font-size", "12px")
  .attr("fill", "black");

  
  // Dessiner les nœuds de chaque arbre (sauf la racine)
  const nodes = treeGroup
  .selectAll(".node")
  .data(tree.descendants())
  .enter()
  .data(tree.descendants().filter(d => d.parent))
  .append("g")
    .attr("class", "node")
    .attr("transform", (d) => `translate(${d.x},${d.y})`)
    .attr("class", function(d) {
    if (!d.parent && d.data.accuracy >= 0.8) {
      return "node high-accuracy";
    } else if (!d.parent && d.data.accuracy >= 0.6) {
      return "node medium-accuracy";
    } else if (!d.parent) {
      return "node low-accuracy";
    } else {
      return "node black";
    }
  })
    .on("mouseover", function(d) {
      tooltip.transition()
        .duration(200)
        .style("opacity", .9);
      tooltip.html(d.data.name)
        .style("left", (d3.event.pageX) + "px")
        .style("top", (d3.event.pageY - 28) + "px");
    })
    .on("mouseout", function(d) {
      tooltip.transition()
        .duration(500)
        .style("opacity", 0);
    })
    .on("click", function(d) {
      const node = d3.select(this);
      const isToggled = node.classed("toggled");

      // Remove the "toggled" class from all nodes in all trees
      svg.selectAll(".node").classed("toggled", false);

      // Toggle the "toggled" class on the clicked node and same nodes
      node.classed("toggled", !isToggled);
      toggleSameNodes(d);
    });
  
  
  // Dessiner des cercles pour représenter les nœuds.
  nodes.append("circle")
    .attr("r", function(d) {
        return 10;
  })
    .attr("fill", function(d) {
      if (d.data.accuracy >= 0.8) {
        return "red";
      } else if (d.data.accuracy >= 0.6) {
        return "blue";
      } else {
        return "black";
      }
    })
    .attr("data-name", d => d.data.name);


  // Ajout d'une infobulle pour afficher le nom du nœud.
  const tooltip = d3.select("body").append("div")
    .attr("class", "tooltip")
    .style("opacity", 0);

  // Dessiner les liens entre les nœuds.
  const links = treeGroup
    .selectAll(".link")
    .data(tree.links())
    .enter()
    .append("path")
      .attr("class", "link")
      .attr("d", d3.linkVertical().x(d => d.x).y(d => d.y))
      .attr("stroke", "black")
      .attr("fill", "none");
  
  // Mise à jour des variables de positionnement et de taille.
  xOffset += maxTreeWidth + margin.right;
  maxHeight = Math.max(maxHeight, maxTreeHeight);
});

//****************************************************
//****************************************************

// Ajustement de la largeur et de la hauteur de la zone de dessin si nécessaire.
const maxGroupLength = Math.max(yOffsetGroup1.length, yOffsetGroup2.length, yOffsetGroup3.length);
const totalHeight = maxGroupLength * (maxHeight + margin.bottom) * verticalSpacingFactor + margin.top + margin.bottom;
if (totalWidth > width) {
  svg.attr("width", totalWidth);
}
if (totalHeight > height) {
  svg.attr("height", totalHeight);
}

// Création de l'axe de l'accuracy.
const accuracyAxis = d3.axisBottom(accuracyScale);


// Ajout de l'axe de l'accuracy en bas de la zone de dessin.
svg.append("g")
  .attr("class", "accuracy-axis")
  .attr("transform", `translate(${margin.left}, ${margin.top*5})`)
  .call(accuracyAxis);

// Ajout d'un titre pour l'axe de l'accuracy.
svg.append("text")
  .attr("class", "accuracy-axis-title")
  .attr("x", totalWidth / 2)
  .attr("y", totalHeight - margin.bottom / 2)
  .attr("text-anchor", "middle");


// Mettez à jour la position X du titre de l'axe de l'accuracy
svg.select(".accuracy-axis-ti